#ifndef VCA_NET_H
#define VCA_NET_H

#include <can_vca.h>
#include <vcasdo_fsm.h>
#include <ul_gavl.h>
#include <ul_list.h>
#include <ul_evcbase.h>
#include <ul_htimer.h>

struct vcaFdNodeCan_t;
struct vcaNet_t;
struct vcaSDOFsm_t;
struct vcaSDORequest_t;

typedef struct vcaNetId_t
{
  int refcnt; /* no used yet */
  int dead:1;
  int masked:1;
  int distinct:1;
  struct vcaNet_t *net;
  union{
    ul_list_node_t list;
    gavl_node_t distinct;
  }node;
  canmsg_id_t  id;
  canmsg_id_t  id_mask; /* CAN mask, 1 bits in mask select the important bits of field id */
  evc_tx_hub_t tx_hub;
}vcaNetId_t;


/** 
 * struct vcaNet_t - structure representing one can network/interface
 * @vcaFdNodeCan_t:	pointer to default CAN interface to the physical bus
 * @can_nodes:		network nodes/devices
 * @distinct_ids:	distinc COBs tree
 * @masked_ids:		masked COBs list
 */
typedef struct vcaNet_t
{
  struct vcaFdNodeCan_t *canfdnode;
  gavl_cust_root_field_t can_nodes; /* tree of CANopen nodes in this net (vcaCANNode_t) */
  gavl_cust_root_field_t distinct_ids; /* tree of CAN IDs currently used in this net (vcaNetId_t) */
  ul_list_head_t masked_ids; /* tree of CAN IDs ranges defined by bit masks (vcaNetId_t) */
  evc_rx_hub_t rx_hub;
} vcaNet_t;

typedef struct vcaSDOFsm_t
{
  evc_rx_hub_t rx_hub;
  vcasdo_fsm_t fsm;
  ul_list_head_t requests;  /* list of pending SDO requests for this SDOFSM */
  struct vcaCANNode_t *cannode;
  /*struct vcaNet_t *net;  use cannode->net instead */
  struct vcaSDORequest_t *actrq;
  ul_htim_diff_t timeout_time;
  ul_htimer_t timer;
  int fsm_connected:1;
} vcaSDOFsm_t;


typedef struct vcaSDORequest_t
{
  ul_list_node_t queue_node;
  vcaSDOFsm_t *sdofsm; /* parent vcaSDOFsm_t */
  
  unsigned int is_upload:1;
  unsigned int is_pending:1;
  uint8_t node;
  uint16_t index;
  uint8_t subindex;
  ul_dbuff_t data;

  int state;
  int err_no;

  void (*finished_callback)(struct vcaSDORequest_t *);
} vcaSDORequest_t;

typedef struct vcaCANNode_t
{
  gavl_node_t tree_node;
  int node_number; /* CANopen node number */
  struct vcaSDOFsm_t sdofsm;
  struct vcaNet_t *net; /* parent vcaNet_t */
} vcaCANNode_t;


vcaNet_t *vcaNet_init(vcaNet_t *net);
void vcaNet_destroy(vcaNet_t *net);
struct vcaFdNodeCan_t* vcaNet_connect_device(vcaNet_t *net, struct vcaFdNodeCan_t *canfdnode, const char *device_name);
vcaNetId_t *vcaNet_find_id(vcaNet_t *net, canmsg_id_t id, int create);
vcaNetId_t *vcaNet_find_masked_id(vcaNet_t *net, canmsg_id_t id, canmsg_id_t id_mask, int create);
vcaCANNode_t *vcaNet_find_cannode(vcaNet_t *net, int node_number, int create);
int vcaNet_msg_rec_connect(vcaNet_t *net, evc_rx_hub_t *rx_hub, canmsg_id_t id, canmsg_id_t id_mask);
int vcaNet_send_can_msg(vcaNet_t *net, const canmsg_t *msg);


vcaNetId_t *vcaNetId_init(vcaNetId_t *netid, canmsg_id_t id, canmsg_id_t id_mask);

vcaCANNode_t *vcaCANNode_init(vcaCANNode_t *cannode, int node_number);
void vcaCANNode_destroy(vcaCANNode_t *cannode);
int vcaCANNode_addSDORequest(vcaCANNode_t *cannode, vcaSDORequest_t *req); 
//vcaSDOFsm_t *vcaCANNode_get_sdofsm(vcaCANNode_t *cannode);

vcaSDOFsm_t *vcaSDOFsm_init(vcaSDOFsm_t *sdofsm, vcaCANNode_t *cannode);
void vcaSDOFsm_destroy(vcaSDOFsm_t *sdofsm);
int vcaSDOFsm_connect(vcaSDOFsm_t *sdofsm);
int vcaSDOFsm_addSDORequest(vcaSDOFsm_t *sdofsm, vcaSDORequest_t *req);
void vcaSDOFsm_processPendingRequest(vcaSDOFsm_t *sdofsm);
void vcaSDOFsm_finishRequest(vcaSDOFsm_t *sdofsm);

vcaSDORequest_t *vcaSDORequest_init(vcaSDORequest_t *req, 
    int is_upload,
    int index, int subindex,
    void *data, int data_len);
void vcaSDORequest_destroy(vcaSDORequest_t *req);
void vcaSDORequest_abort(vcaSDORequest_t *req);

#endif /*VCA_NET_H*/
